package client.driver;

import java.math.BigInteger;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.Vector;

import common.Commands;


import client.model.Communicator;
import client.model.AnonymousUser;
import client.model.HelpDeskUser;
import client.listener.AnonymousListener;
import client.listener.ServerListener;
import client.listener.ConnectListener;
import client.listener.HelpDeskListener;
import client.listener.LoginListener;
import client.view.AnonymousLoginWindow;
import client.view.AnonymousWindow;
import client.view.ConnectWindow;
import client.view.HelpDeskWindow;
import client.view.LoginWindow;

public class Anonymous implements ServerListener {

	public static int windows = 0;	
	protected Communicator communicator;

	private ConnectListener connectListener;
	private ConnectWindow connectWindow;
	protected LoginListener loginListener;
	protected LoginWindow loginWindow;
	protected Vector<AnonymousListener> anonymousListeners;
	protected Vector<AnonymousWindow> anonymousWindows;

	public Anonymous() {		
		this.communicator = new Communicator();
		communicator.addClientListener(this);
		this.connectListener = new ConnectListener(communicator);
		this.connectWindow = new ConnectWindow(connectListener);
		this.loginListener = null;
		this.loginWindow = null;
		this.anonymousWindows = new Vector<AnonymousWindow>();
		this.anonymousListeners = new Vector<AnonymousListener>();
	}

	public void start() {
		connectWindow.setVisible(true);
	}

	public void initLoginWindow() {
		loginWindow = new AnonymousLoginWindow(loginListener);
	}

	public void switchToLoginWindow() {
		connectWindow.setVisible(false);
		connectListener = null;
		connectWindow = null;
		loginListener = new LoginListener(communicator);
		initLoginWindow();
		loginWindow.setVisible(true);
	}

	public void switchToChatWindow() {
		loginWindow.setVisible(false);
		loginListener = null;
		loginWindow = null;
		AnonymousUser user = communicator.getUser();
		AnonymousListener listener;

		if (user != null && user instanceof HelpDeskUser) {	
			listener = new HelpDeskListener(communicator);			
			HelpDeskWindow window = new HelpDeskWindow((HelpDeskListener) listener);
			anonymousListeners.add(listener);
			anonymousWindows.add(window);
		} else if (user instanceof AnonymousUser) {
			listener = new AnonymousListener(communicator);			
			anonymousListeners.add(listener);
			anonymousWindows.add(new AnonymousWindow(listener));
		} else {
			// user is either null or of an unsupported type	
			return;
		}

		setWindowsVisible(true);
	}

	public void setWindowsVisible(boolean isVisible) {
		AnonymousWindow window;
		Iterator<AnonymousWindow> it = anonymousWindows.iterator();

		while (it.hasNext()) {
			window = it.next();
			window.setVisible(true);			
		}
	}

	public void closeChatWindow() {
		setWindowsVisible(false);
		anonymousListeners = null;
		anonymousWindows = null;
	}


	@Override
	public void newUser(String other) {
		//Not needed
	}

	@Override
	public void removeUser(String other) {
		AnonymousWindow window;
		AnonymousListener listener;
		Iterator<AnonymousWindow> it = anonymousWindows.iterator();
		String id;

		while (it.hasNext()) {
			window = it.next();
			id = window.getUser().getId();
			if (id != null && id.equals(other)) {
				window.setVisible(false);
				listener = window.getListener();
				anonymousListeners.remove(listener);
				anonymousWindows.remove(window);
				break;
			}
		}		
	}

	@Override
	public void serverResponse(String command, String options) {
		try {			
			StringTokenizer tokenizer;
			String status;

			if (options != null) {
				tokenizer = new StringTokenizer(options);
			} else {
				tokenizer = null;
			}

			if (command.equals(Commands.CONNECTION_REPLY)) {
				status = tokenizer.nextToken();

				if (status.equals(Commands.LOGIN_SUCCESS)) {
					communicator.setConnectionStatus(true);
					switchToLoginWindow();
				}						
			} else if (command.equals(Commands.NOTIFY)) {
				if (tokenizer != null && tokenizer.countTokens() >= 2) {
					String type = tokenizer.nextToken();					
					AnonymousWindow window;
					Iterator<AnonymousWindow> it = anonymousWindows.iterator();

					if (type.equals(Commands.ASSIGNED)) {
						String user = tokenizer.nextToken();

						while (it.hasNext()) {
							window = it.next();
							if (window.getUser().getId() == null) {
								window.getUser().setId(user);
								break;
							}
						}								
					} else if (type.equals(Commands.LOGGED_OUT)) {
						String id = tokenizer.nextToken();

						while (it.hasNext()) {
							window = it.next();
							if (window.getUser().getId().equals(id)) {
								window.getUser().setId(null);
								window.getUser().setName(null);
								break;
							}
						}				
					} else {
						//Unknown notification type
					}

				} else {
					// Server notify has too few tokens
				}
			} else if (command.equals(Commands.MESSAGE)) {
				if (tokenizer != null && tokenizer.countTokens() >= 1) {
					String id = tokenizer.nextToken();
					String message = options.replace(id, "").trim();

					if (id.equals(Commands.SERVER)) {	
						boolean added = false;
						AnonymousWindow aw;
						Iterator<AnonymousWindow> it = anonymousWindows.iterator();
						while (it.hasNext()) {
							aw = it.next();
							added = true;
							if (aw.getUser().getId() == null) {
								aw.addText(message);
								break;
							}
						}

						if (!added) {
							anonymousWindows.firstElement().addText(message);
						}
					} else {
						AnonymousWindow window;
						Iterator<AnonymousWindow> it = anonymousWindows.iterator();
						String username;

						while (it.hasNext()) {
							window = it.next();
							if (window.getUser().getId() != null && window.getUser().getId().equals(id)) {
								window.addText(message);
								username = message.substring(0, message.indexOf(" ")-1);

								//Hack to display title of person you are talking to
								if (window.getUser().getName() != null && 
										username.equals(window.getUser().getName())) {
									window.setTitle("Chat with " + username);
								}
								window.getUser().setName(username);

								break;
							}
						}
					}
				} else {
					//Message recipient not specified
				}
			} else if (command.equals(Commands.LOGIN_REPLY)) {
				if (tokenizer != null && tokenizer.hasMoreTokens()) {
					status = tokenizer.nextToken();

					if (status.equals(Commands.LOGIN_SUCCESS)) {
						String id = tokenizer.nextToken();

						AnonymousUser user =  communicator.getUser();

						if (user != null) {
							user.setId(id);
							user.login();
						} else {
							//error user object is null
							//continue;
						}

						switchToChatWindow();
					} else {
						// unknown server login response
					}
				} else {
					//Login reply doesn't return whether login successful
				}
			} else if(command.equals(Commands.NOTIFY)) {
				if (tokenizer != null && tokenizer.hasMoreTokens()) {
					status = tokenizer.nextToken();
					String message;

					if (status.equals(Commands.LOGGED_OUT)) {
						if (tokenizer.hasMoreTokens()) {
							message = "Your chat session has been terminated.";
							String id = tokenizer.nextToken();
							Iterator<AnonymousWindow> it = anonymousWindows.iterator();
							AnonymousWindow aw;

							while (it.hasNext()) {
								aw = it.next();

								if (aw.getUser().getId().equals(id)) {
									aw.addText(message);
									aw.getUser().setId(null);
									break;
								}
							}					
						}
					}
				}

			} else {
				// unrecognised response
			}			
		} catch (Exception e) {
			//unexpected error
		}
	}

	public String [] removeUsername(String [] array, String id) {
		Vector<String> vector = new Vector<String>(array.length);

		for (int i = 0; i < array.length; i++) {
			if (!id.equals(array[i])) {
				vector.add(array[i]);
			}
		}

		return (String []) vector.toArray();
	}	

	public static void main(String [] args) {
		Anonymous chat = new Anonymous();
		chat.start();
	}
}
