package client.driver;

import java.io.IOException;
import java.util.Iterator;
import java.util.StringTokenizer;

import javax.swing.JOptionPane;

import common.Commands;
import common.Common;
import common.Error;


import client.listener.HelpDeskListener;
import client.view.AnonymousWindow;
import client.view.HelpDeskLoginWindow;
import client.view.HelpDeskWindow;

public class HelpDesk extends Anonymous {

	public HelpDesk() {
		super();
	}

	public void initLoginWindow() {
		loginWindow = new HelpDeskLoginWindow(loginListener);
	}

	public void serverResponse(String command, String options) {

		try {
			StringTokenizer tokenizer = null;

			if (options != null) {
				tokenizer = new StringTokenizer(options);
			}

			if (command.equals(Commands.LOGIN_REPLY) && 
					(options.indexOf(Commands.LOGIN_FAILURE) > -1 || 
							options.indexOf(Commands.LOGIN_DENIED) > -1)) {
				loginWindow.setMessage(Error.WRONG_PASSWORD);
				loginWindow.pack();
				loginWindow.repaint();
			} else if (command.equals(Commands.TRANSFER_LIST_REPLY) && 
					anonymousWindows.firstElement() instanceof HelpDeskWindow) {
				String id = communicator.getUser().getId();
				String [] users = Common.tokensToArray(tokenizer, id);		
				HelpDeskWindow hdw = null;
				AnonymousWindow aw;			
				String anonymousId = null;
				Iterator<AnonymousWindow> it = anonymousWindows.iterator();

				while (it.hasNext()) {
					aw = it.next();
					if (aw instanceof HelpDeskWindow) {
						hdw = (HelpDeskWindow) aw;
						if (hdw.transferRequested()) {
							anonymousId = hdw.getUser().getId();
							break;
						}						
					}
				}

				if (hdw != null && hdw.transferRequested()) {
					if (users != null && users.length > 0) {
						String helpDeskUser = (String)JOptionPane.showInputDialog(
								hdw,
								"Which help desk user do you want to transfer this chat to?",
								"Help Desk User Transfer Dialog",
								JOptionPane.QUESTION_MESSAGE,
								null,
								users,
								users[0]
								);

						if (helpDeskUser == null) {
							hdw.addText("Help Desk Transfer request CANCELLED");
						} else {
							communicator.transferUser(helpDeskUser, anonymousId);
						}
					} else {
						//There are no available users to transfer this chat to
					}
				} else {
					//There is no help desk window
				}				
			} else if (command.equals(Commands.TRANSFER_REPLY)) {
				if (tokenizer.hasMoreTokens()) {
					String status = tokenizer.nextToken();
					Iterator<AnonymousWindow> it = anonymousWindows.iterator();
					AnonymousWindow aw;
					HelpDeskWindow hdw;
					String message;

					if (status.equals(Commands.SUCCESS)) {
						message = "Anonymous user was successfully transferred";
						while (it.hasNext()) {
							aw = it.next();
							if (aw instanceof HelpDeskWindow) {
								hdw = (HelpDeskWindow) aw;
								if (hdw.transferRequested()) {
									hdw.setTransferRequested(false);
									hdw.addText(message);
									hdw.getUser().setId(null);
									break;
								}
							}
						}
					} else if (status.equals(Commands.FAILURE)) {
						message = "Anonymous User transfer was unsuccessful";

						while (it.hasNext()) {
							aw = it.next();
							if (aw instanceof HelpDeskWindow) {
								hdw = (HelpDeskWindow) aw;
								if (hdw.transferRequested()) {
									hdw.setTransferRequested(false);
									hdw.addText(message);
									break;
								}
							}
						}
					}
				}				
			} else if (command.equals(Commands.GET_LIST_REPLY)) {
				AnonymousWindow aw;
				HelpDeskWindow hdw;
				Iterator<AnonymousWindow> it = anonymousWindows.iterator();
				String id = communicator.getUser().getId();
				String [] array;

				while (it.hasNext()) {
					aw = it.next();
					if (aw instanceof HelpDeskWindow) {
						hdw = (HelpDeskWindow) aw;
						if (hdw.listRequested()) {
							array = Common.tokensToArray(tokenizer, id);
							hdw.setListRequested(false);
							hdw.setUsers(array);
							break;
						}						
					}
				}
			} else if (command.equals(Commands.MESSAGE) && !recipientExists(options)) {
				String id = tokenizer.nextToken();
				String message = options.replace(id, "").trim();

				newUser(id);

				Iterator<AnonymousWindow> it = anonymousWindows.iterator();
				AnonymousWindow aw;

				while (it.hasNext()) {
					aw = it.next();

					if (aw.getId().equals(id)) {
						aw.addText(message);
						break;
					}
				}
			} else {
				//Command must be in the anonymous scope
				if(command.equals(Commands.NOTIFY)) {
					if (tokenizer != null && tokenizer.hasMoreTokens()) {
						String status = tokenizer.nextToken();
						String message;						
						AnonymousWindow aw;
						HelpDeskWindow hdw;
						Iterator<AnonymousWindow> it;

						if (status.equals(Commands.ASSIGNED) && tokenizer.hasMoreTokens()) {	
							boolean found = false;
							String id = tokenizer.nextToken();
							it = anonymousWindows.iterator();
							HelpDeskWindow noAssignedUser = null;
							HelpDeskWindow requestedUser = null;

							while (it.hasNext()) {
								aw = it.next();

								if (aw instanceof HelpDeskWindow) {
									hdw = (HelpDeskWindow) aw;
									if (hdw.userRequested() && hdw.getUser().getId() == null) {
										//hdw.setUserRequested(false);
										found = true;
										requestedUser = hdw;
										requestedUser.getUser().setId(id);
										requestedUser.setUserRequested(false);
										return;
									} else if (hdw.getUser().getId() == null) {
										found = true;
										noAssignedUser = hdw;
									} else if (hdw.userRequested()) {
										hdw.setUserRequested(false);
									}									
								}
								/*
								if (aw.getUser().getId() == null) {
									found = true;
									if (aw instanceof HelpDeskWindow) {
										hdw = (HelpDeskWindow) aw;
										if (hdw.userRequested()) {
											hdw.setUserRequested(false);
										}
									}
									break;
								} */
							}	
							
							if (requestedUser == null && noAssignedUser != null) {
								noAssignedUser.getUser().setId(id);
								return;
							} else if (!found) {
								newUser(id, true);

								// set all user requested flags to false
								it = anonymousWindows.iterator();

								while (it.hasNext()) {
									aw = it.next();
									if (aw instanceof HelpDeskWindow) {
										hdw = (HelpDeskWindow) aw;
										hdw.setUserRequested(false);
									}
								}
								return;
							}
						} else if (status.equals(Commands.NO_USER)) {
							message = "No users are currently available.";
							it = anonymousWindows.iterator();

							while (it.hasNext()) {
								aw = it.next();

								if (aw instanceof HelpDeskWindow) {
									hdw = (HelpDeskWindow) aw;
									if (hdw.userRequested()) {
										hdw.addText(message);
										hdw.setUserRequested(false);
										break;
									}									
								}
							}

							return;
						} 
					}
				} else if (command.equals(Commands.END_CHAT)) {

				}

				super.serverResponse(command, options);				
			}
		} catch (IOException e) {
			// client server communication error
		} catch (Exception e) {
			//unknown error occureed
		}
	}

	@Override
	public void newUser(String other) {
		newUser(other, false);
	}

	private void newUser(String other, boolean isHelpDeskWindow) {
		AnonymousWindow window;
		Iterator<AnonymousWindow> it = anonymousWindows.iterator();
		boolean hasHelpDesk = false;
		boolean exists = false;
		String id;

		while (it.hasNext()) {
			window = it.next();
			id = window.getUser().getId();
			if (window instanceof HelpDeskWindow) {
				hasHelpDesk = true;
			}
			if (id != null && id.equals(other)) {
				exists = true;
			}
		}

		if (hasHelpDesk && !exists) {
			HelpDeskListener listener = new HelpDeskListener(communicator);
			if (isHelpDeskWindow) {
				window = new HelpDeskWindow(listener, false);
			} else {
				window = new AnonymousWindow(listener, true);
			}

			window.setId(other);
			anonymousListeners.add(listener);
			anonymousWindows.add(window);
			window.setVisible(true);
		}
	}

	private boolean recipientExists(String options) {
		boolean exists = false;
		StringTokenizer tokenizer;

		if (options != null) {
			tokenizer = new StringTokenizer(options);
			String [] params = Common.tokensToArray(tokenizer);
			if (params.length > 1) {
				String recipient = params[0];
				Iterator<AnonymousWindow> it = anonymousWindows.iterator();
				AnonymousWindow aw;

				if (recipient.equals(Commands.SERVER)) {
					exists = true;
				} else {
					while(!exists && it.hasNext()) {
						aw = it.next();
						String id = aw.getId();
						exists = id != null && id.equals(recipient);
					}
				}		
			}
		}	

		return exists;
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub
		HelpDesk chat = new HelpDesk();
		chat.start();
	}
}
