package client.listener;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.IOException;

import javax.swing.JButton;

import common.Error;

import client.model.Communicator;
import client.view.ConnectWindow;


public class ConnectListener implements MouseListener {
	
	private static final String CONNECT_ERROR = "Unable to connect to server";

	private ConnectWindow window;
	
	private Communicator communicator;

	public ConnectListener(Communicator communicator) {
		this.communicator = communicator;
	}

	@Override
	public void mouseClicked(MouseEvent event) {
		/* No need to check which button since there is only one.*/
		
		if (event.getSource() instanceof JButton) {
			try {
				String hostname = window.getUrl();
				int port = Integer.parseInt(window.getPort());
				communicator.connect(hostname, port);
			} catch (IOException e) {
				window.setMessage(CONNECT_ERROR);
			} catch (NumberFormatException e) {
				window.setMessage(CONNECT_ERROR);
			} catch (Exception e) {
				window.setMessage(Error.UNKNOWN_ERROR);
			}
			
			window.pack();
			window.repaint(); 
		}
	}

	@Override
	public void mousePressed(MouseEvent e) {
		// DO NOTHING
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		// DO NOTHING		
	}

	@Override
	public void mouseEntered(MouseEvent e) {
		// DO NOTHING
	}

	@Override
	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub
	}
	
	public void setWindow(ConnectWindow other) {
		this.window = other;
	}

}
