package client.listener;

import java.awt.event.MouseEvent;
import java.io.IOException;

import javax.swing.JButton;

import client.model.Communicator;
import client.view.HelpDeskWindow;

public class HelpDeskListener extends AnonymousListener {

	private final String ACTIVATE_ERROR = "Error changing transfer mode status";
	private final String EXIT_STRING = "!@#$%^&*()";
	private Communicator communicator;

	private HelpDeskWindow window;

	public HelpDeskListener(Communicator communicator) {
		super(communicator);
		this.communicator = communicator;
	}

	@Override
	public void mouseClicked(MouseEvent event) {
		JButton button;

		if(event.getSource() instanceof JButton) {
			button = (JButton) event.getSource();
			String label = button.getText();

			if (label.equals(HelpDeskWindow.END_CHAT_LABEL)) {
				try {
					communicator.endChat(window.getUser().getId());
					window.getUser().setId(null);										
				} catch (IOException e) {
					window.addText("Error ending chat");
				} catch (Exception e) {

				}
			} else if (label.equals(HelpDeskWindow.ACTIVATE_TRANSFER_MODE)) {
				try {
					window.setInTransferMode(true);
					communicator.activateTransferMode();
					window.addText("Your status is now activated");
				} catch (IOException e) {
					window.addText(ACTIVATE_ERROR);
				} catch (Exception e) {

				}
				window.toggleTransferMode();
			} else if (label.equals(HelpDeskWindow.DEACTIVATE_TRANSFER_MODE)) {
				try {
					window.setInTransferMode(false);
					communicator.deactivateTransferMode();
					window.addText("Your status is now deactivated");
				} catch (IOException e) {
					window.addText(ACTIVATE_ERROR);
				} catch (Exception e) {

				}
				window.toggleTransferMode();
			} else if (label.equals(HelpDeskWindow.UPDATE_LABEL)) {
				try {
					window.setListRequested(true);
					communicator.getHelpDeskUsers();
				} catch (IOException e) {
					window.addText("Error retrieving help desk user list");
				} catch (Exception e) {

				}
			} else if (label.equals(HelpDeskWindow.TALK_TO_LABEL)) {
				String helpDeskUser = window.getSelectedUser();

				if (helpDeskUser != null) {
					communicator.triggerNewUser(helpDeskUser);
				} else {
					window.addText("Please select a user in the Help Desk" +
							" User list before clicking the 'Talk To' button");
				}
			} else if (label.equals(HelpDeskWindow.TRANSFER_LABEL)) {

				try {
					window.setTransferRequested(true);					
					communicator.getTransferList();
				} catch (IOException e) {
					window.addText("Error starting transfer of user");
				} catch (Exception e) {

				}
			} else if (label.equals(HelpDeskWindow.REQUEST_LABEL)) {
				try {
					window.setUserRequested(true);
					communicator.requestUser();
				} catch (IOException e) {
					window.addText("Error requesting new user");
				} catch (Exception e) {

				}
			} else if (label.equals(HelpDeskWindow.CLOSE_LABEL)) {
				String id = window.getUser().getId();

				if (id != null) {
					try {
						communicator.endChat(id);
					} catch (IOException e) {
						window.addText("Error requesting new user");
					} catch (Exception e) {

					}
				}
				
				window.getUser().setId(EXIT_STRING);
			}
		}		

		//handle all events in parent
		super.mouseClicked(event);
	}

	@Override
	public void mousePressed(MouseEvent e) {
	}

	@Override
	public void mouseReleased(MouseEvent e) {
	}

	@Override
	public void mouseEntered(MouseEvent e) {
	}

	@Override
	public void mouseExited(MouseEvent e) {
	}

	public void setHelpDeskWindow(HelpDeskWindow window) {
		this.window = window;
	}
}
