package client.view;

import java.awt.BorderLayout;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;

import client.listener.HelpDeskListener;

public class HelpDeskWindow extends AnonymousWindow {
	
	private static final long serialVersionUID = -7215554058649386694L;
	
	public static final String TRANSFER_LABEL = "TRANSFER";
	public static final String TALK_TO_LABEL = "TALK TO";
	public static final String ACTIVATE_TRANSFER_MODE = "ACTIVATE TRANSFER MODE";
	public static final String DEACTIVATE_TRANSFER_MODE = "DEACTIVATE TRANSFER MODE";
	public static final String END_CHAT_LABEL = "END CHAT";
	public static final String UPDATE_LABEL = "UPDATE";
	public static final String USER_LABEL = "HELP DESK USERS";
	public static final String REQUEST_LABEL = "REQUEST NEW USER";
	public static final String CLOSE_LABEL = "CLOSE";
	
	private String [] users;	
	private JList<String> userList;
	private JButton activeTransfer;
	private JButton updateUsers;
	private boolean transferRequested;
	private boolean listRequested;
	private boolean userRequested;
	private boolean inTransferMode;
	private HelpDeskListener listener;
	
	public HelpDeskWindow(HelpDeskListener listener) {
		this(listener, true);
	}
	
	
	public HelpDeskWindow(HelpDeskListener listener, boolean isMainWindow) {
		super(listener, !isMainWindow);
		
		JToolBar toolbar = new JToolBar();
		JButton transfer = new JButton(TRANSFER_LABEL);
		JButton talkTo = new JButton(TALK_TO_LABEL);
		activeTransfer = new JButton(ACTIVATE_TRANSFER_MODE);
		JButton endChat = new JButton(END_CHAT_LABEL);
		JButton requestUser = new JButton(REQUEST_LABEL);
		
		this.listener = listener;
		this.users = null;
		this.userList = new JList<String>();
		this.transferRequested = false;
		this.listRequested = false;
		this.userRequested = false;
		this.inTransferMode = false;
		
		listener.setHelpDeskWindow(this);
		
		talkTo.addMouseListener(listener);
		transfer.addMouseListener(listener);
		activeTransfer.addMouseListener(listener);
		endChat.addMouseListener(listener);
		requestUser.addMouseListener(listener);
		
		toolbar.add(transfer);
		toolbar.add(talkTo);
		toolbar.add(activeTransfer);
		toolbar.add(requestUser);
		toolbar.add(endChat);
		
		this.add(toolbar, BorderLayout.NORTH);
		
		JPanel panel = new JPanel(new BorderLayout());
		JLabel userHeading = new JLabel(USER_LABEL);
		updateUsers = new JButton(UPDATE_LABEL);
		
		updateUsers.addMouseListener(listener);
		
		panel.add(userHeading, BorderLayout.NORTH);
		panel.add(updateUsers, BorderLayout.SOUTH);
		panel.add(new JScrollPane(userList), BorderLayout.CENTER);
		
		this.add(panel, BorderLayout.WEST);
	}
	
	public void toggleTransferMode() {		
		activeTransfer.setText(activeTransfer.getText().equals(ACTIVATE_TRANSFER_MODE) ? DEACTIVATE_TRANSFER_MODE : ACTIVATE_TRANSFER_MODE);
	}
	
	public String getSelectedUser() {
		int index = userList.getSelectedIndex();
		return index != -1 ? users[index] : null;
	}
	
	public void setUsers(String[] users) {
		this.users = users;
		userList.setListData(users);
		this.repaint();
	}

	public boolean transferRequested() {
		return transferRequested;
	}

	public void setTransferRequested(boolean transferRequested) {
		this.transferRequested = transferRequested;
	}

	public boolean listRequested() {
		return listRequested;
	}

	public void setListRequested(boolean listRequested) {
		this.listRequested = listRequested;
	}

	public boolean userRequested() {
		return userRequested;
	}

	public void setUserRequested(boolean userRequested) {
		this.userRequested = userRequested;
	}

	public boolean isInTransferMode() {
		return inTransferMode;
	}

	public void setInTransferMode(boolean inTransferMode) {
		this.inTransferMode = inTransferMode;
	}

	public HelpDeskListener getListener() {
		return listener;
	}
	
}
