package server;
import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;

import java.net.Socket;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.util.StringTokenizer;

import javax.crypto.BadPaddingException;
import javax.crypto.IllegalBlockSizeException;

import common.MessageDigest;


public class ServerThread extends Thread
{
		/*********************************************************************
  						Communication Command Protocols
		 *********************************************************************/
	final String CMD_NOTIFY		= "cmd_notify: ";						//from server side as cmd + ASSIGNED + username
																							//cmd + LOGGED_OUT + username
																							//cmd + NO_USER + username
	final String ASSIGNED		= "ASSIGNED ";
	final String LOGGED_OUT		= "LOGGED_OUT ";
	final String NO_USER		= "NO_USER ";
	final String CMD_CON  		= "cmd_connection: ";					//from server side as cmd + msg   msg can be OK or BUSY
	final String CMD_LOGIN 		= "cmd_login: ";						//from client side as cmd + type + username + password
	final String CMD_LOGIN_MSG	= "cmd_login_msg: ";					//from server side as cmd + msg       msg can be FAIL or DENY or OK + ID/username
	final String CMD_GET_LIST 	= "cmd_get_list: ";						//from client side as cmd
	final String CMD_SENT_LIST 	= "cmd_sent_list:";						//from server side as cmd + username1 + username2 + ....
	final String CMD_END_CHAT	= "cmd_end_chat: ";						//from client side as cmd + username1
	final String CMD_LOGOUT		= "cmd_logout: ";						//from server side as cmd
	final String CMD_GET_TRANSFER_LIST	= "cmd_get_transfer_list: ";	//from client side as cmd
	final String CMD_SENT_TRANSFER_LIST	= "cmd_sent_transfer_list: ";	//from server side as cmd + username1 +username2 ...
	final String CMD_TRANSFER_TO        = "cmd_transfer_to: ";			//from client side as cmd + HelpDesk_username + Anonyumous_username		
	//final String CMD_ACTIVATE	= "cmd_activate: ";						//from server side as cmd + options      // options can be either 0 or 1
	final String CMD_MSG		= "cmd_msg: ";							//from client or server side message as  cmd + to-username + message
	final String CMD_REQUIST    = "cmd_request_user: ";					//from client side as cmd
	final String CMD_TRANSFER_MODE    = "cmd_transfer_mode: ";			//from client side as cmd + options      // options can be either 0 or 1
	final String TRANSFER_REPLY = "cmd_transfer_reply: "; 				//from client side as cmd + SUCCESS or FAILURE
	
	
	//*************Protocol Ends  **************
	
	private final int CLIENT = 0;
	private final int HDESK = 1;
	private Socket connected_Socket;
	private CServer server;
	
	private DataOutputStream data_to_currentuser;
	private BufferedReader data_from_client;
	
	private String current_username ="";
	private String current_FName = "";
	
	private String assigned_client_username ="";
	private String assigned_hDesk_username ="";
	
	private String statement = "";
	private int user_type;

	
	public ServerThread(CServer server, Socket connectedSocket) throws Exception
	{
		this.server = server;
		this.connected_Socket = connectedSocket;
		this.data_to_currentuser = new DataOutputStream(connected_Socket.getOutputStream());	
		this.data_from_client =  new BufferedReader(new InputStreamReader(connected_Socket.getInputStream()));		
		//DataOutputStream out = new DataOutputStream(connected_Socket.getOutputStream());
			
	}
	public void run()
	{
		try
		{
			boolean loop_control = true;
			
			statement = CMD_CON + "OK" +"\n"; // Send connection confirmation to client side
			send_msg(statement);
				
			
			/*********************************************************************
			  	The thread keep listening a client side request in while loop
			 *********************************************************************/
			
			while(loop_control)
			{
				
				String msgFromClient = new String();
		
				msgFromClient = data_from_client.readLine();
				StringTokenizer st = new StringTokenizer(msgFromClient);	//messages are broken into words
				String command = (st.hasMoreTokens())? (st.nextToken()).trim(): "";		//get the first word from the client side message
				
				/*****************************************
						  Log In request
				*****************************************/
				if(command.equals(CMD_LOGIN.trim()))
				{
					String temp_type = "";
					String temp_username = "";
					String temp_pass = "";
					String[] msg = this.msg_break(st);		//get the first three words in an Array
					
					if(msg[0] != null)
					{
						temp_type = msg[0];					//get the user type from the client side
						
						/******************
						 Login as Help Desk
						 ******************/
						if((msg[1] != null) && (msg[2] != null) && (temp_type.equals("1")))  
						{
							
							temp_username = msg[1];			//get the user name of help desk
							temp_pass = msg[2];				//get the password
							if(server.login_check(temp_username, temp_pass))
							{
								if(server.chat_dir_username_exsit(temp_username))		// check if help desk already logged in at another terminal
								{
										statement = CMD_LOGIN_MSG + "DENY" + "\n";  //  login deny because this guy already logged in on another terminal
										send_msg(statement);
									loop_control = false;			// Stop the while loop and end the Thread
								}
								else
								{
									current_FName = server.get_HelpD_FName(temp_username);
									ChatDirectory temp_dir = new ChatDirectory(temp_username, current_FName, Integer.parseInt(temp_type), data_to_currentuser);
									server.add_chat_dir(temp_dir);	// add the data on chat directory
									this.user_type = Integer.parseInt(temp_type);
									this.current_username = temp_username;
										statement = CMD_LOGIN_MSG + "OK" + " " + current_username + " " + "\n";		//Login sucessfully
										send_msg(statement);
										statement = CMD_MSG + "SERVER" + " Welcome to Company XYZ. Please wait, You will be assigned to the next customer \n";
										send_msg(statement);
									

									//server.helpdesk_assignment(current_username, (CMD_NOTIFY + ASSIGNED), CMD_MSG); // assign the help desk 
								}								
							}
							
							else { 
								statement = CMD_LOGIN_MSG + "FAIL" + "\n";
								send_msg(statement);
								}		// login fail					
						}
						
						/***********************
						 Login as Anonymous User
						 ***********************/
						else if((msg[1] != null) && (temp_type.equals("0")) && (msg[2] == null))  		
						{
							if (!server.anonymous_exist_on_chat_dir(this.data_to_currentuser))
							{
								this.current_FName = msg[1];		//get a name
								this.current_username = (new Integer(server.create_client_ID())).toString();	// Issue a unique username ID for Anonymous User
								ChatDirectory temp_dir = new ChatDirectory(current_username, current_FName, Integer.parseInt(temp_type), data_to_currentuser);
								server.add_chat_dir(temp_dir);		// add data on chat directory
								this.user_type = Integer.parseInt(temp_type);
									statement = CMD_LOGIN_MSG + "OK" + " " + current_username +"\n";
									send_msg(statement);
									statement = CMD_MSG + "SERVER" + " Welcome to Company XYZ. Please wait, the next help desk will be assigned shortly. \n";
									send_msg(statement);	
									
								//server.anonymous_user_assignment(current_username, (CMD_NOTIFY + ASSIGNED), CMD_MSG);  // assign an anonymous user
							}
							else
							{
								// the Anonymous user already registered on chat directory. No need to issue user ID
							}
							
						}
						
						else { 
							statement = CMD_LOGIN_MSG + "FAIL" + "\n";
							send_msg(statement);
							}// login fail
					}
					
					else {
						statement = CMD_LOGIN_MSG + "FAIL" + "\n";
						send_msg(statement);
						}// login fail
				}
				
				
				/****************************************************************************
				  				Chattting message communication
				 ****************************************************************************/
				else if(command.equals(CMD_MSG.trim()))  // chatting between client and help desk, vise versa
				{
					String sentence = current_FName + ": " + this.get_message(msgFromClient, 1);  // out going message

					if(st.hasMoreTokens())  
					{
						String chatt_partner_username = st.nextToken();		// get the next word from communication protocol--- coming from client side
						
						if(user_type == CLIENT)		// If this thread owns by Anonymous user
						{
							String temp_username = server.get_chat_dir(current_username).get_assigned_username();		// get the chat partner from the chat direcotry
							if(temp_username != null)	// check the chat partner Help desk user name is not null
							{
								assigned_hDesk_username = temp_username;
									statement = CMD_MSG + current_username + " " + sentence + "\n";
									send_msg(server.get_chat_dir(assigned_hDesk_username).get_CurrentDatastrem(), statement);
								}
							else
							{
								statement = CMD_MSG + "SERVER" + " Currently, there is no help desker available for help. Please try later..." + "\n";
								send_msg(statement);
								}						
						}
						else if(user_type == HDESK)	// If this Thread owns by Help Desk
						{
							if(server.chat_dir_username_exsit(chatt_partner_username))	// check if it does exist in the chat directory
							{
								if(server.get_chat_dir(chatt_partner_username).get_userType() == 1)  // check if this person is Help Desk
								{
									statement = CMD_MSG + current_username + " " + sentence + "\n";
									send_msg(server.get_chat_dir(chatt_partner_username).get_CurrentDatastrem(), statement);
									}
								else if((server.get_chat_dir(chatt_partner_username).get_userType() == 0) && server.get_chat_dir(current_username).check_client_list(chatt_partner_username))  // check if this person is Anonymous user and also check if it is on the list
								{
									assigned_client_username = chatt_partner_username;
									statement = CMD_MSG + current_username + " " + sentence + "\n";
									send_msg(server.get_chat_dir(assigned_client_username).get_CurrentDatastrem(), statement);
									}
								else
								{
									statement = CMD_MSG + "SERVER" + " Currently, you haven't been assigned to client. Please wait..." + "\n";
									send_msg(statement);
									}
							}
							else
							{
								//data_to_currentuser.writeBytes(CMD_MSG + "SERVER" + " Message Not deliver, most probably your client quit the chat session" + "\n");
							}
						}
					}
					else
					{
						//data_to_currentuser.writeBytes(CMD_MSG + "SERVER" + "Please provide the destination where your message is going to deliver. \n");
					}
				}		
				//===================================================		
				
				
				/****************************************************************************
  							End Chat Request from HELP DESK or ANONYMOUS USER
				 ****************************************************************************/
				
				else if(command.equals(CMD_END_CHAT.trim()))		// chat end between client and help desk, vise versa
				{
					
					if(user_type == HDESK)
					{
						if(st.hasMoreTokens())
						{
							String temp_name = st.nextToken();
							if((server.chat_dir_username_exsit(temp_name)))
							{
								if(server.get_chat_dir(current_username).check_client_list(temp_name))
								{
									assigned_client_username = temp_name;
									server.get_chat_dir(current_username).remove_client_list(assigned_client_username);
									try
									{
										server.get_chat_dir(assigned_client_username).set_activate(0);
											statement = CMD_NOTIFY + LOGGED_OUT + current_username  + "\n";
											send_msg(server.get_chat_dir(assigned_client_username).get_CurrentDatastrem(), statement);
											statement = CMD_MSG + "SERVER" + " Thank you for visiting us. Good bye..." + "\n";
											send_msg(server.get_chat_dir(assigned_client_username).get_CurrentDatastrem(), statement);	
										server.get_chat_dir(assigned_client_username).set_assigned_username(null);
										server.get_chat_dir(assigned_client_username).set_status(0);
									}
									catch(Exception e) {}  // in case the user first close its connection
									statement = CMD_MSG + "SERVER" + " You have finished the chat session with your client." + "\n";
									send_msg(statement);
								}
							}							
						}
						
						//server.helpdesk_assignment(current_username, (CMD_NOTIFY + ASSIGNED), CMD_MSG);  // assign the help desk
					}
				}
				
				/****************************************************************************
							TRANSFER LIST Request from Help Desk
				 ****************************************************************************/
				
				else if(command.equals(CMD_GET_TRANSFER_LIST.trim()))
				{
					if(user_type == HDESK)
					{
						String HDesk_lists = server.get_Transfer_To_HelpD_List();
						if(HDesk_lists != null)	// get the list
						{
							statement = CMD_SENT_TRANSFER_LIST + HDesk_lists + "\n";
							send_msg(statement);
							}
						else
						{
							statement = CMD_SENT_TRANSFER_LIST + "\n";
							send_msg(statement);
							}
						if(server.get_chat_dir(current_username).total_client_list() == 0)  // inform the help desk that he has no client to transfer to other
						{
							statement = CMD_MSG + "SERVER" + " You do not have a client to transfer to another Help Desk." + "\n";
							send_msg(statement);
							}
					}
					else
					{
						//data_to_currentuser.writeBytes(CMD_MSG + "SERVER" + " You are not a help desk. Please close the chat enivronment." + "\n");
					}
				}				
				
				/****************************************************************************
								Initiate the Transfer
				 ****************************************************************************/
				
				else if(command.equals(CMD_TRANSFER_TO.trim()))		//Help desk is transfering a client to another help desk
				{
					if(user_type == HDESK)
					{
						if(server.get_chat_dir(current_username).total_client_list() != 0)		// check if it has assigned clients
						{
							if(st.hasMoreTokens())				
							{
								String transfered_HDesk_username = st.nextToken();				// get the other Help Desk username
								if(server.chat_dir_username_exsit(transfered_HDesk_username))	// check if the other Help Desk exist on chat directory
								{	// if the other Help Desk party is on transfer mode
									if(server.get_chat_dir(transfered_HDesk_username).get_userType() == 1 && server.get_chat_dir(transfered_HDesk_username).get_transfer_mode() == 1)	
									{
										if(st.hasMoreTokens())
										{
											String client_username = st.nextToken();
											if(server.chat_dir_username_exsit(client_username) && server.get_chat_dir(current_username).check_client_list(client_username))
											{
																								
												statement = CMD_NOTIFY + "LOGGED_OUT " + current_username + "\n";
												send_msg(server.get_chat_dir(client_username).get_CurrentDatastrem(), statement);
												
												statement = CMD_NOTIFY + "ASSIGNED " + transfered_HDesk_username + "\n";
												send_msg(server.get_chat_dir(client_username).get_CurrentDatastrem(), statement);
												
												statement = CMD_NOTIFY + "ASSIGNED " + client_username + "\n";
												send_msg(server.get_chat_dir(transfered_HDesk_username).get_CurrentDatastrem(), statement);
												
												statement = TRANSFER_REPLY + "SUCCESS " + "\n";
												send_msg(statement);
												
												statement = CMD_MSG + transfered_HDesk_username + " You are now transfered to another Help Desk for better assistance. Thanks for beeing with us." + "\n";
												send_msg(server.get_chat_dir(client_username).get_CurrentDatastrem(), statement);
												
												server.get_chat_dir(transfered_HDesk_username).add_client_list(client_username);  // add this transfered anonymous user to the helpD list
												
												statement = CMD_MSG + client_username + " You have a transfered client waiting for you. Please start chatting." + "\n";
												send_msg(server.get_chat_dir(transfered_HDesk_username).get_CurrentDatastrem(), statement);
												
																								
												server.get_chat_dir(client_username).set_assigned_username(transfered_HDesk_username);
												
												
												//statement = CMD_MSG + "SERVER" +  " Please wait, You will be assigned to the next customer." + "\n";
												//send_msg(statement);
												server.get_chat_dir(current_username).remove_client_list(client_username);

												//server.helpdesk_assignment(current_username, CMD_NOTIFY, CMD_MSG);  // assign the help desk
											}
											else
											{
												statement = TRANSFER_REPLY + "FAILURE"+ "\n";
												send_msg(statement);
												}
										}
									}
									else
									{
										statement = TRANSFER_REPLY + "FAILURE" + "\n";
										send_msg(statement);
										}
								}
								else
								{
									statement = TRANSFER_REPLY + "FAILURE" + "\n";
									send_msg(statement);
									//data_to_currentuser.writeBytes(CMD_MSG + "SERVER" + " This help desk is not available at this time. Try anothe one. " + "\n");
								}
							}
							else
							{
								statement = TRANSFER_REPLY + "FAILURE" + "\n";
								send_msg(statement);
								//data_to_currentuser.writeBytes(CMD_MSG + "SERVER" + " Please select Help Desk username to transfer. " + "\n");
							}
						}
						else
						{
							statement = TRANSFER_REPLY + "FAILURE" + "\n";
							send_msg(statement);
							}
					}
					else
					{
						statement = TRANSFER_REPLY + "FAILURE" + "\n";
						send_msg(statement);
						//data_to_currentuser.writeBytes(CMD_MSG + "SERVER" + " You are not a help desk. Please close the chat enivronment." + "\n");
					}
				}
				
				/****************************************************************************
							request of all logged in HELP DESK usernames
				 ****************************************************************************/
				
				else if(command.equals(CMD_GET_LIST.trim()))
				{
					if(user_type == HDESK)
					{
						if(server.get_all_HDesk_userName() != null)
						{
							statement = CMD_SENT_LIST + server.get_all_HDesk_userName() + "\n";
							send_msg(statement);
							}
						else
						{
							statement = CMD_SENT_LIST  + "\n";
							send_msg(statement);
							}
					}
					else
					{
						//data_to_currentuser.writeBytes(CMD_MSG + "SERVER" + " You are not a help desk. Please close the chat enivronment." + "\n");
					}
					
				}
				
				/****************************************************************************
								Receiving REQUEST from Help Desk for assigning an Anonymous user
				 ****************************************************************************/
				
				else if(command.equals(CMD_REQUIST.trim()))
				{
					if(user_type == HDESK)
					{
						server.get_chat_dir(current_username).set_request(1);
						if(!server.helpdesk_assignment(current_username, (CMD_NOTIFY + ASSIGNED), CMD_MSG)) // if it is ready and free, the server will assign the next waiting customer 
						{
							statement = CMD_NOTIFY  + NO_USER + "\n";
							send_msg(statement);
						}
						server.get_chat_dir(current_username).set_request(0);
					}
					else
					{
						//data_to_currentuser.writeBytes(CMD_MSG + "SERVER" + " You are not a help desk. Please close the chat enivronment." + "\n");
					}
				}
				
				/****************************************************************************
						TRANSFER MODE ON/OFF status for Help Desk who is getting a transferred Anonymous user
				 ****************************************************************************/
				
				else if(command.equals(CMD_TRANSFER_MODE.trim()))
				{
					if(user_type == HDESK)
					{
						if(st.hasMoreTokens())
						{
							String msg = st.nextToken();		// get the option
							if(msg.equals("1") || msg.equals("0"))
							{
								server.get_chat_dir(current_username).set_transfer_mode(Integer.parseInt(msg));
							}							
						}						 
					}					
				}
				
				/****************************************************************************
							LOG OUT request from Help Desk
				 ****************************************************************************/
				
				else if(command.equals(CMD_LOGOUT.trim()))  
				{
					String temp_username = server.get_chat_dir(current_username).get_assigned_username();	// get chat partner user name
					if(user_type == CLIENT)
					{
						if(temp_username != null)
						{
							server.get_chat_dir(current_username).set_activate(0);  // Syncronization is solved. the anomunous user is no longer use the service. 
							server.get_chat_dir(temp_username).remove_client_list(current_username);
							assigned_hDesk_username = temp_username;
							
							try
							{
								statement = CMD_NOTIFY + LOGGED_OUT + current_username  + "\n";
								send_msg(server.get_chat_dir(assigned_hDesk_username).get_CurrentDatastrem(), statement);
								
								}
							catch(Exception e){} //in case the help desk close its connection.
							statement = CMD_MSG + "SERVER" + " Thank you for visiting us. Good bye..." + "\n";
							send_msg(statement);
							
							server.get_chat_dir(current_username).set_assigned_username(null);
							server.get_chat_dir(current_username).set_status(0);
							server.remove_user(current_username);
							this.connected_Socket.close();
		// Do we need to shut down the while loop????  					
						}
						else
						{
							if (server.chat_dir_username_exsit(current_username))
							{
								server.get_chat_dir(current_username).set_activate(0);  // Syncronization is solved. the anomunous user is no longer use the service. 
								server.remove_user(current_username);
								this.connected_Socket.close();
							}
							else
							{
								this.connected_Socket.close();
							}
							
						}
					}
					else if(user_type == HDESK)
					{
						server.get_chat_dir(current_username).set_request(0);  // for synchronization
						
						this.assigned_client_username = server.get_chat_dir(current_username).get_assigned_username();
						
						int total_client =server.get_chat_dir(current_username).total_client_list();
						if(total_client != 0) // the help desk hasn't finished serving his clients
						{
							for(int i = 1; i <= total_client; i++)		// go throw in the client list
							{
								assigned_client_username = server.get_chat_dir(current_username).remove_client_list(0);	//remove it from the list
								if(server.chat_dir_username_exsit(assigned_client_username))
								{
									server.get_chat_dir(assigned_client_username).set_status(0); // let the client wait for the next help desk.
									
									statement = CMD_MSG + "SERVER" + " Please wait, the next help desk will help you sortly." + "\n";
									send_msg(server.get_chat_dir(assigned_client_username).get_CurrentDatastrem(), statement);
									
									//server.anonymous_user_assignment(assigned_client_username, (CMD_NOTIFY + ASSIGNED), CMD_MSG);  // assign an anonymous user
								}
							}							
						}
						statement = "Thank you. Have a good day.   bye bye.... " + "\n";
						send_msg(statement);
						server.remove_user(current_username);
						this.connected_Socket.close();						
					}
					
					loop_control = false;
				}
			}
			
		}catch (Exception e)
			{
				/****************************************************************************
							unexpected termination by the HELP DESK or ANONYMOUS USER
				 ****************************************************************************/
				
				if(user_type == CLIENT)		// the client has lost its connection
				{
					if(server.chat_dir_username_exsit(current_username))
					{
						server.get_chat_dir(current_username).set_activate(0);   // for synchronization
						assigned_hDesk_username = server.get_assigned_username(current_username);		// get the assigned held desk username
						if(assigned_hDesk_username != null)
						{	try{
							if(server.get_chat_dir(assigned_hDesk_username).check_client_list(current_username)) // check if it is in the list
							{
								server.get_chat_dir(assigned_hDesk_username).remove_client_list(current_username); // remove it from the list
							}
							}catch(Exception k){}
						}
						
						if(assigned_hDesk_username != null && server.has_client(current_username))		// and this client was in the middle of chatting
						{
											
							if(server.chat_dir_username_exsit(assigned_hDesk_username))		// check if the help desk is in chat directory list
							{
								try
								{
									statement = CMD_MSG + "SERVER" + " Your client is no longer available." + "\n";
									send_msg(server.get_chat_dir(assigned_hDesk_username).get_CurrentDatastrem(), statement);
									
									//server.helpdesk_assignment(assigned_hDesk_username, (CMD_NOTIFY + ASSIGNED), CMD_MSG);
								}
								catch(Exception g) {}
							}							
						}
						try
						{
							this.connected_Socket.close();
						}
						catch(Exception f) {}
						
						server.remove_user(current_username);
					}
				}
				else if(user_type == HDESK)
				{
					if(server.chat_dir_username_exsit(current_username))
					{
						server.get_chat_dir(current_username).set_request(0);   // for synchronization
						int total_client =server.get_chat_dir(current_username).total_client_list();						
						if(total_client != 0)		// and check help desk if he was in the middle of chating
						{
							for(int i = 1; i <= total_client; i++)		// go throw in the client list
							{
								assigned_client_username = server.get_chat_dir(current_username).remove_client_list(0);	//remove it from the list
								if(server.chat_dir_username_exsit(assigned_client_username))
								{
									server.get_chat_dir(assigned_client_username).set_assigned_username(null);
									server.get_chat_dir(assigned_client_username).set_status(0);	// change the status to free			
									try {
									statement = CMD_MSG + "SERVER" + " Sorry your chatting session just disconnected. Please wait the next help desk will be here shortly." + "\n";
									send_msg(server.get_chat_dir(assigned_client_username).get_CurrentDatastrem(), statement);
									
									//server.anonymous_user_assignment(assigned_client_username, (CMD_NOTIFY + ASSIGNED), CMD_MSG);
									} catch(Exception k) {}
								}
							}							
						}
						try
						{
							this.connected_Socket.close();
						}
						catch(Exception f) {}
						
						server.remove_user(current_username);
					}
				}
			}
		server.thread_out();	// it deduct one from thread counter
	}
	
	/*
	 * This method reads the first three words from StringTokenizer and
	 * returns with string of array.
	 */
	public String[] msg_break(StringTokenizer st)
	{
		String[] ary = new String[3];
		for(int i = 0; i < 3; i++)
		{
			if(st.hasMoreTokens())
			{
				ary[i] = st.nextToken().trim();
			}
			else
			{
				ary[i] = null;
			}
				
		}
		
		return ary;
	}
	
	/*
	 * This method cut out the command and chat partner user name  from the message
	 */
	public String get_message(String message, int type)  // it returns a message with out a command protocol
	{
		String msg = "";
		StringTokenizer st = new StringTokenizer(message);
		
		if(type == 0 && st.hasMoreTokens()) // chat between Anonymous user-to-Help Desk or vise versa
		{
			msg = st.nextToken();		// this is a communication protocol
			msg = (st.hasMoreTokens())? (message.substring(msg.length() + 1, message.length())) : "";     //this is the message
			return msg;			
		}
		if(type == 1)					// chat between helpdesk-to-helpdesk
		{
			if(st.hasMoreTokens())
			{
				st.nextToken();		// this is a communication  protocol
			}
			if(st.hasMoreTokens())
			{
				st.nextToken();		// this is a help desk partner user name
			}
			while(st.hasMoreTokens())
			{
				msg = msg + st.nextToken() + " ";		// this is the message.
			}
			return msg;
		}
		
		return message;
	}
	
	public void send_msg(String stat) throws IOException
	{
		data_to_currentuser.writeBytes(stat);
	}
	public void send_msg(DataOutputStream out, String stat) throws IOException 
	{
		out.writeBytes(stat);
	}
}
